/*******************************************************************************/
/*                                                              28.9.2010      */    
/* Testprogramm fuer Multitasking                                              */
/*                                                                             */
/* Das Programm laesst nur zwei LEDs blinken.                                  */
/*                                                                             */
/* Zum Programmieren des Controllers muss der Jumper auf der                   */
/* Platine gesetzt sein, das Programm wird dagegen nur gestartet               */
/* wenn kein Jumper gesteckt ist                                               */
/*                                                                             */
/* Die Kommunikation findet mit dem Port1 des M16C statt, nicht                */
/* mit Port0!                                                                  */
/*                                                                             */
/* Der M16C62A (M30624FGAFP) hat 256kb Flash und 20kb Ram                      */
/* Achtung, nur 192kb Flash nutzbar, Linkerscript lesen!                       */
/*******************************************************************************/
#define MAIN
#include <stdarg.h>
#include "sfr62.h"         // special funktion register
#include "System.h"

#include "interrupt.h"     // Interrupttabelle
#include "ticker.h"        // Interruptfunktion fuer Zeitgeber
#include "tools.h"         // Allgemeiner Kram

#include "taskerC.h"       // Unser Multitasker

#include "MultiTasker.h"


/*******************************************************************************/
/* Das ist die Mainfunction der ersten Task                                    */
/*******************************************************************************/
void main_task1(void)
{ 

  do
    {

      Semphore_wait(SEMA_PORT);
      LED2_off;
      Semaphore_free(SEMA_PORT);
      delay(500);

      Semphore_wait(SEMA_PORT);
      LED2_on;
      Semaphore_free(SEMA_PORT);
      delay(500);
 

    }
  while(1);

}

/*******************************************************************************/
/* Das ist die Mainfunction der zweiten Task                                   */
/*******************************************************************************/
void main_task2(void)
{ 

  do
    {
      Semphore_wait(SEMA_PORT);
      LED1_on; 
      Semaphore_free(SEMA_PORT);
      delay(300);
       
      Semphore_wait(SEMA_PORT);
      LED1_off; 
      Semaphore_free(SEMA_PORT);
      delay(300);

      
    }
  while(1);


}

/*******************************************************************************/
/* Hauptprogramm                                                               */
/*******************************************************************************/
int  main()
{  

   maxspeed();              /* Prozessor auf 16Mhz schalten                  */
   init_delay_timer();      /* DelayTimer initialisieren                     */
   init_ticker();           /* Initialisiert den Ticker-Interrupt    	     */
   install_irq_tbl();       /* Schaltet die Interrupts ein!                  */

   tasker_init();           /* Multitasker initialisieren starten */

   /* Led-Ports auf Ausgang schalten */
   LED1_init; LED2_init;
   LED1_off;  LED2_off;


   /* Installiert unsere Tasks */
   tasker_func(&main_task1,15,0);
   tasker_func(&main_task2,5,1);

   /* Startet das Multitasking */
   tasker_start();

   /* Unnoetig, aber sonst wirft der gcc immer eine Warnung raus */
   while(1);
}


