#include "sfr62.h"    // special funktion register
#include "interrupt.h"

#include "ticker.h"  /* Wir brauchen die Deklarationen fuer die */
#include "taskerC.h"

typedef void (*vector_table_type)(void);

/* Definiert einen Dummyeintrag fuer ungenutzte IRQs    */
/* So eine Zeile muss es auch fuer jeden genutzten IRQ  */
/* geben damit der Compiler den richtigen Code erzeugt. */
void __attribute__ ((interrupt   )) dummyint(void);

/* Diese Funktion sollte eigentlich niemals aufgerufen werden! */
void dummyint(void)
{
}

/*******************************************************************/
/* Dies ist eine Interrupttabelle die im Flashrom liegt!           */
/* Es ist aber NICHT die normale Tabelle am ende des Flashroms     */
/* Das hat den Vorteil das wir bei einem Fehler nicht die ID-Bytes */
/* der CPU ueberschreiben.                                         */
/*******************************************************************/
const vector_table_type int_vecs[]  =
  {
    dummyint, 0,         // 0: BRK
    dummyint, 0,         // 1: (reserved)
    dummyint, 0,         // 2: (reserved)
    dummyint, 0,         // 3: (reserved)
    dummyint, 0,         // 4: INT3
    dummyint, 0,         // 5: Timer B5
    dummyint, 0,         // 6: Timer B4
    dummyint, 0,         // 7: Timer B3
    dummyint, 0,         // 8: SIO4 oder INT5
    dummyint, 0,         // 9: SIO3 oder INT4
    dummyint, 0,         // 10: Bus colision
    dummyint, 0,         // 11: DMA0
    dummyint, 0,         // 12: DMA1
    dummyint, 0,         // 13: Key Input
    dummyint, 0,         // 14: A/D Converter
    dummyint, 0,         // 15: UART2 transmit
    dummyint, 0,         // 16: UART2 recieve
    dummyint, 0,         // 17: UART0 transmit
    dummyint, 0,         // 18: UART0 receive
    dummyint, 0,         // 19: UART1 transmit
    dummyint, 0,         // 20: UART1 receive
    dummyint, 0,         // 21: Timer A0
    tickerint,0,         // 22: Timrt A1
    dummyint, 0,         // 23: Timer A2
    dummyint, 0,         // 24: Timer A3
    tasker_switch, 0,         // 25: Timer A4
    dummyint, 0,         // 26: Timer B0
    dummyint, 0,         // 27: Timer B1
    dummyint, 0,         // 28: Timer B2
    dummyint, 0,          // 29: INT0
    dummyint, 0,         // 30: INT1
    dummyint, 0,         // 31: INT2
  };



/*******************************************************************/
/* Installiert die Interrupttabelle und schaltet alle IRQs ein     */
/*******************************************************************/
void install_irq_tbl(void)
{
  asm("fclr i"); /* Interrupts ausschalten */


  // Initialisierung der Interruptvektor-Tabelle:
  asm("ldc #_int_vecs, intbl");
  asm("ldc #0,         intbh");

  asm("fset i"); /* Interrupts einschalten */

}
