	/************************************************************/
	/* Es ist ein TimerIRQ aufgetreten und wir starten hier den */
	/* Scheduler als Interruptfunktion.                         */
	/*                                                          */
	/* Beim Aufruf liegen folgende 4Byte auf dem Stack:         */
	/*                                                          */
	/* SP+0  Middle und Low Byte des PCs                        */
	/* SP+2  Flags Low                                          */
	/* SP+3  Flags High + PC-High des unterbrochenen Programms  */
        /*                                                          */
	/* Diese Funktion setzt den zur Task gehoerenden            */
	/* Registersatz und springt dann ueber den reit-Befehl      */
	/* in die jeweilige Task                                    */
	/************************************************************/

		.text
	        .global _tasker_switch
	        .type _tasker_switch, @function

_tasker_switch:
	pushc   fb /* Nur das noetigste sichern */
	pushm	a1
	
	/* Wir setzen den FramePointer auf unser TaskRam */
	mov.w	#_TaskPointer,a1	/* Adresse eines Zeiger        */
	ldc     [a1],fb

	
	/* Wenn eine Task frisch gestartet wird dann duerfen wir */
	/* ihre Register nicht sichern/ueberschreiben            */
	cmp.w	#1,28[fb]
	jeq   _ts1
	
	/* Sichert den Registersatz der letzten Task in das TaskRam */
	mov.w	r0,0[fb]		/* R0 */
	mov.w	r1,2[fb]		/* R1 */
	mov.w	r2,4[fb]		/* R2 */
	mov.w	r3,6[fb]		/* R3 */
	mov.w	a0,8[fb]		/* A0 */
	popm    a1
	mov.w	a1,10[fb]		/* A1 */
	popm    a1                      /* Holt Frambuffer! */
	mov.w	a1,12[fb]		/* FB */
	stc	sb,14[fb]      		/* SB */	
	stc	sp,16[fb]		/* SP = Stack */

	
	/* Jetzt muessen wir den PC sichern! */
	mov.w   0[sp],22[fb]		/* Middle und Lowbyte der Adresse */
	mov.w	2[sp],18[fb]      	/* Highnibble und Flags           */	
	jmp.w	_ts2

	/* Es wird eine Task frisch gestartet */
_ts1:   popm    a1			/* Stack bereinigen A1 */
	popm	a1                      /* Stack bereinigen fb */
_ts5:	mov.w 	#2,28[fb]               /* TASK_RUN!           */
	jmp.w	_ts4

_ts2:
	
	/* Jetzt haben wir nur noch die Ruecksprungadresse auf    */
	/* dem Stack und alle Register sind im TaskRam gesichert  */

	
	/* TaskPointer auf das TaskRam der naechsten Task setzen */
_ts3:	mov.w	#_TaskPointer,a1   	/* Position von TaskPointer    */
        mov.w  20[fb],[a1]		/* TaskRam[].next -> TaskPointer */
	ldc	[a1],fb			/* FramePointer anpassen  */

	/* Wenn eine Task auf Prio=0 ist, dann die naechste Task holen! */
	cmp.w	#0,26[fb]		/* TASK_START ? */
	jeq   _ts3

	/* Sonderbehandlung wenn Task erstmalig gestartet wird */
	cmp.w	#1,28[fb]
	jeq   _ts5

	
_ts4:

	/* Im Kontextswitch rufen wir noch eine C-Funktion auf */
/*	jsr.w	_tasker_main */

	mov.w  #0x038e,a0		/* Zaehlregister Timer A4 */
	mov.w  24[fb],[a0]		/* Laufzeit der Task laden */
	
	/* Den Stack der Task setzen */
	ldc     16[fb],sp  		/* SP = Stack */	

	/* Jetzt muessen wir die Adresse fuer den Ruecksprung auf      */
	/* den Stack schaffen. Dadurch wird die Task am Ende mit       */
	/* mit reit gestartet                                          */
	mov.w   22[fb],0[sp]            /* Middle und Lowbyte der Adresse */
	mov.w   18[fb],2[sp]		/* Highnibble und Flags */
	
	/* Holt den Registersatz aus dem  TaskRam */	
	mov.w	0[fb],r0		/* R0 */
	mov.w	2[fb],r1		/* R1 */
	mov.w	4[fb],r2		/* R2 */
	mov.w	6[fb],r3		/* R3 */
	mov.w	10[fb],a1		/* A1 */
	ldc     14[fb],sb   		/* SB */
	mov.w	8[fb],a0		/* A0 */
	ldc	12[fb],fb		/* FB = FrameBuffer */		
	
	reit

