/*******************************************************************************/
/*                                                       Olix V1.0  6.10.2010  */
/* Erster Versuch eines kleinen Multitaskers                                   */
/*                                                                             */
/* Immer dran denken, der M16C ist Little-Endian, also wie Intel!              */
/*                                                                             */
/* Es wird hier ein praemptives Multitasking installiert. Es gibt eine         */
/* garantierte Rundenzeit in der jede Task garantiert einmal laeuft.           */
/* Die Rechnenzeit einer Task wird ansonsten anhand der Verhaeltnisses der     */
/* der Prioritaeten zugeteilt.                                                 */
/*                                                                             */
/* Man darf nicht vergessen das einzelne C-Befehle damit nicht mehr atomar     */
/* sind. Dies ist besonders wichtig beim Zugriff auf einzelne Bits eines Ports */
/* Es muss sichergestellt sein das in der Zeit keine andere Task auf diesen    */
/* Port zugreift!                                                              */
/* Man sollte sich auch gut ueberlegen ob man auf Systemfunktionen             */
/* zurueckgreifen kann oder nicht.                                             */
/*                                                                             */
/* Sinnvolle Verbesserungen:                                                   */
/*                                                                             */
/* Es waere ganz nett wenn man die Stackgroesse pro Task einstellen koennte    */
/*                                                                             */
/*******************************************************************************/

#include "sfr62.h"         // special funktion register
#include "System.h"

#include "interrupt.h"     // Interrupttabelle
#include "ticker.h"        // Interruptfunktion fuer Zeitgeber
#include "tools.h"         // Allgemeiner Kram
#include "MultiTasker.h"

#include "taskerC.h"

unsigned char TaskCounter; //Zaehlt die Anzahl der Tasks

Semaphor SemaList[MAX_SEMA]={ {1}, {1}, {1} };  

/*******************************************************************************/
/* Taskliste verketten                                                         */
/*******************************************************************************/
void tasker_datainit(void)
{ int i;

  /* Ein paar ungefaehrliche Werte setzen */
  for (i=0;i<(MAX_TASKS-1);i++)
    {
      TaskRam[i].next = (void *) &(TaskRam[i+1]); 
      TaskRam[i].Adresse    = NULL;
      TaskRam[i].Prioritaet = 0;
      TaskRam[i].RunFlag    = TASK_STOP;
    }
  TaskRam[(MAX_TASKS-1)].next = (void *) &(TaskRam[0]);

  /* Wir fangen mit der ersten Task in der Liste an */
  TaskPointer = (void *) &(TaskRam[0]);

  TaskCounter = 0;
}

/*******************************************************************************/
/* Initialisiert einen Timer damit wir einen IRQ bekommen wo die Task          */
/* umgeschaltet werden                                                         */
/* Das geschieht vorerst 1000mal pro Sekunde                                   */
/*                                                                             */
/* Da der Interrupt sehr haeufig laufen muss, ist es wichtig das er eine       */
/* niedrige Prioritaet hat.                                                    */
/*******************************************************************************/
void tasker_init(void)
{

  tasker_datainit();

  ta4mr = 0x80; /* Timer mode mit 16Mhz / 32 = 500khz Takt */
  ta4ud = 0;    /* Wir wollen runterzaehlen */

  ta4s = 0;                 /* Timer anhalten                         */
  ir_ta4ic = 0;             /* Ueberlaufflag im IRQ-Register loeschen */
  ta4  = 500;              /* Zaehlvariable setzen                   */

  ta4ic = 0x01;             /* IRQ auf Level1, (niedrig) freigeben    */

  /* Der Timer ist jetzt bereit, aber laeuft noch nicht! */
}


/***********************************************************************/
/* Berechnet anhand der Prioritaeten die Laufzeiten der einzelen Tasks */
/***********************************************************************/
void tasker_priocalc(void)
{ int i;
  int AllPrio=0;
  int PrioBase;

  /* Maximale Prioritatensumme */
  for (i=0;i<TaskCounter;i++) AllPrio += TaskRam[i].Prioritaet;

  /* Wertigkeit eines Priopunktes berechnen */
  PrioBase = (RoundTimer/2)/AllPrio;

  /* In Zeiteinheiten fuer die Task umrechnen */
  for (i=0;i<TaskCounter;i++) TaskRam[i].TimeValue = TaskRam[i].Prioritaet * PrioBase;

}

/***********************************************************************/
/* Ab hier laufen die einzelnen Tasks                                  */
/* Die Funktion kehrt niemals wieder zurueck!                          */
/***********************************************************************/
void tasker_start(void)
{
  tasker_priocalc();
  ta4s = 1;                 /* Timer starten                          */
  while(1);                 /* Point of no Return                     */
}

/***********************************************************************/
/* Diese Funktion traegt die im Multitask laufenden Funktionen         */
/* in die Taskliste ein                                                */
/*                                                                     */
/* *function   = 16Bit Adresse der Funktion                            */
/* Prioritaet  = 0=aus, >0 ist die Prioritaet der Funktion             */
/* TaskNum     = Position in der Taskliste                             */
/*                                                                     */
/* Der M16C arbeitet eigentlich mit 24Bit Adressen. Leider kann der    */
/* gcc aber nur 16bit Adressen. Damit der Compiler trotzdem alle       */
/* Funktionen im 24Bit Adressraum erreichen kann, legt er eine Tabelle */
/* mit absoluten Spruengen in den ersten 64k des Programms ab.         */
/* Diese Tabelle liegt also im RAM des Prozessor und wird dort vom     */
/* Startupcode hinkopiert! Das bedeutet im uebrigen auch das jede      */
/* Function im Programm 4Byte Ram belegt und das wir nur Speicher mit  */
/* maximal 64kByte Groesse verwalten koennen.                          */
/*                                                                     */
/* Wir muessen hier nun die Adresse der Task-Funktion speichern        */
/* damit wir die Task erstmalig starten koennen. Im weiteren Programm  */
/* ablauf werden dann immer die echten 24Bit-Adressen gespeichert      */
/* an denen sich der PC gerade befindet.                               */
/* Fuer den Start wird aber die 16Bit Adresse im Ram erwartet!         */
/* Die 16Bit Adresse wird dann hier zur 24Bit Adresse erweitert. Das   */
/* kein Problem weil wir ja wissen wo sich das Ram befindet.           */
/*                                                                     */
/* Die Laufzeiten einer Task werden anhand des Verhaeltnisses der      */
/* Prioritaeten untereinander berechnet. Darauf achten das die Zahlen  */
/* bei der Berechnung noch in das 16Bit Zaehlregister passen muessen   */
/***********************************************************************/
void tasker_func(void * Function, unsigned char Prioritaet, unsigned char TaskNum)
{ 

  if (TaskNum > MAX_TASKS-1) return;

  /* Den Taskspeicher auf brauchbare Anfangswerte setzen */
  TaskRam[TaskNum].Adresse       = Function;      /* 16Bit Funktionsadresse speichern */
  TaskRam[TaskNum].Prioritaet    = Prioritaet;    /* Prioritaet der Funktion          */ 
  TaskRam[TaskNum].RegR0         = 0;
  TaskRam[TaskNum].RegR1         = 0;
  TaskRam[TaskNum].RegR2         = 0;
  TaskRam[TaskNum].RegR3         = 0;
  TaskRam[TaskNum].RegA0         = 0;
  TaskRam[TaskNum].RegA1         = 0;
  TaskRam[TaskNum].RegFB         = 0;
  TaskRam[TaskNum].RegSB         = 0;

  /* Jede Task bekommt ihren eigenen Stack.               */
  /* Jeder Stack benoetig 4Byte OverTop weil wir die Task */
  /* im Interrupt ueber die Ruecksprungadresse des reit   */
  /* starten!                                             */
  /* Ausserdem waechst der Stack natuerlich zu kleineren  */
  /* Adressen                                             */
  TaskRam[TaskNum].RegSP         = (unsigned short) (TaskRam[TaskNum].stack) 
                                   + STACK_SIZE -4;

  /* Flagregister setzen, enthaelt Flags UND Adresse Highnibble */
  /* Was bedeutet 0x0041:                                       */
  /*3: 0  Highnibble des Flagregisters, IPL=0                   */
  /*2: 0  Highnibble der 24Bit Funktionsadresse immer Null!     */
  /*1: 4  Midnibble  des Flagregisters, I=1                     */
  /*0: 1  Lownibble  des Flagregisters, C=1                     */
  TaskRam[TaskNum].RegFLG        = 0x0041;

  /* Task soll frisch gestartet werden */
  TaskRam[TaskNum].RunFlag    = TASK_START;

  TaskRam[TaskNum].TimeValue  = 50000;
  TaskCounter++;
}

/***********************************************************************/
/* Damit wird die eigene Task angehalten                               */
/***********************************************************************/
void tasker_stop(void)
{ TaskMemory *tpointer;

  asm("fclr i"); /* Interrupts ausschalten */
  tpointer = TaskPointer;
  asm("fset i"); /* Interrupts einschalten */

  tpointer->Prioritaet=0;

}

/***********************************************************************/
/* Task gibt restliche Laufzeit ab                                     */
/***********************************************************************/
void tasker_suspend(void)
{

  asm("fclr i");          /* Interrupts ausschalten                 */
  ta4s = 0;               /* Timer anhalten                         */
  if (ta4>5) ta4 = 5;     /* Kleines bisschen Restlaufzeit          */
  ta4s = 1;               /* Timer starten                          */
  asm("fset i");          /* Interrupts einschalten                 */

}

/***********************************************************************/
/* Wir bleiben hier solange drin bis wir die Semaphore gegrapscht haben*/
/***********************************************************************/
void Semphore_wait(int SemaValue)
{

  asm("fclr i"); /* Interrupts ausschalten */
  SemaList[SemaValue].counter -=1;
  asm("fset i"); /* Interrupts einschalten */


  while(SemaList[SemaValue].counter<0)  tasker_suspend(); 

}

/***********************************************************************/
/* Semaphore wieder freigeben                                          */
/***********************************************************************/
void Semaphore_free(int SemaValue)
{

  asm("fclr i"); /* Interrupts ausschalten */
  SemaList[SemaValue].counter +=1;
  asm("fset i"); /* Interrupts einschalten */

}

/***********************************************************************/
/* Diese Funktion verlaesst sich darauf das sie 100mal pro Sekunde     */
/* von einem Interrupt aufgerufen wird. Das wird benoetigt wenn man    */
/* delay-funktion nutzen will die in jeder Task aufgerufen werden kann */
/***********************************************************************/
void tasker_delaycount(void)
{ int i;

  for (i=0;i<TaskCounter;i++) 
    if (TaskRam[i].DelayCount>0) TaskRam[i].DelayCount--;

}

/***********************************************************************/
/* Wartet solange bis die uebergebene Zeit in ms abgelaufen ist.       */
/* Es gibt eine minimale Wartezeit und eine Granularitaet von 10ms     */
/* Das laesst sich einfach anpassen wenn der delaycount haeufiger      */
/* aufgerufen wird. Aber normalerweise sollten 10ms ausreichen und das */
/* haelt die Interruptlast schoen niedrig                              */ 
/***********************************************************************/
void delay(unsigned short timevalue)
{ TaskMemory *tpointer;

  asm("fclr i"); /* Interrupts ausschalten */
  tpointer = TaskPointer;
  asm("fset i"); /* Interrupts einschalten */

  tpointer->DelayCount= timevalue/10;

  /* Wenn wir warten dann geben wir unsere Rechenzeit ab      */
  /* Das macht das Timing leicht ungenau, aber ist natuerlich */
  /* besser fuer das Systemverhalten                          */
  while(tpointer->DelayCount>0)  tasker_suspend(); 
  

}

/***********************************************************************/
/* Wird derzeit nicht genutzt. Koennten wir im taskerA aktivieren und  */
/* dann jedesmal aufrufen wenn zwischen den Tasks umgeschaltet wird    */
/***********************************************************************/
void tasker_main(void)
{

}
